#include "gomd-qt-example.h"

using namespace std;


int string2int(const string s) {
 if (s.length() == 0) {
  return -1;
  } else {
  return(atoi(s.c_str()));
  }
}


void Tokenize(const string& str,vector<string>& tokens,const string& delimiters = " ")
{
    // Skip delimiters at beginning.
    string::size_type lastPos = str.find_first_not_of(delimiters, 0);
    // Find first "non-delimiter".
    string::size_type pos     = str.find_first_of(delimiters, lastPos);

    while (string::npos != pos || string::npos != lastPos)
    {
        // Found a token, add it to the vector.
        tokens.push_back(str.substr(lastPos, pos - lastPos));
        // Skip delimiters.  Note the "not_of"
        lastPos = str.find_first_not_of(delimiters, pos);
        // Find next "non-delimiter"
        pos = str.find_first_of(delimiters, lastPos);
    }
}


// constructor
qtgomd::qtgomd()
{
 setCaption("gomd-qt-example");
 first=1;
}

// destructor
qtgomd::~qtgomd()
{
}


void qtgomd::setgomdip(string gip) {
 gomdip=gip;
}




void qtgomd::starttimer() {
 refreshtimer = new QTimer( this, "intern" );
 QObject::connect( refreshtimer, SIGNAL(timeout()), this, SLOT(refresh()) );
 refreshtimer->start(5000);
 refresh();
}


int qtgomd::refresh()
{
	//if no value was inserted at the command line, we try to get the load of
	//a node, i.e. the 7th.
	string remoteGomdIP;
	string remoteGomdPort;
	int h=1;
	int labelheight=25;
	int myheight=50;
	remoteGomdIP = gomdip;
	remoteGomdPort =  "9889";


	/*
		1)Open the library (libgomd.so)
	*/
	// cout << "- C++ CLASS LOADING EXAMPLE" << endl;
	// cout << "- Trying to open the library..." << endl;
	void* handle = dlopen("/usr/lib/libgomd.so.1.0.1", RTLD_LAZY);
	if (!handle)
	{
		cerr << "- Cannot load library: " << dlerror() << endl;
		return EXIT_FAILURE;
	}


	/*
		2)Load symbols to create/destroy the instances of libgomd classes
	*/
	//cout << "- Trying to load symbols from the library..." << endl;
	create_t* create_libgomd = (create_t*) dlsym(handle, "create");
	destroy_t* destroy_libgomd = (destroy_t*) dlsym(handle, "destroy");

	/*
		3)Manage symbols errors (if these symbols are not found,
		  an error will be raised)
	*/
	if (!create_libgomd || !destroy_libgomd)
	{
		cerr << "- Cannot load symbols: " << dlerror() << endl;
		return EXIT_FAILURE;
	}

	/*
		4)All went right: print a message.
	*/
	// cout << "- Ok: all symbols were found and loaded successfully!!!" << endl;


	/*
		5)Now we create an instance of the class libgomd
	*/
	libiface* lg = create_libgomd();

	/*
		6)Now we get the load of a specific node
		Please notice the different approach:
		- m_id is included in lgomd.hpp
		- getLoad() is implemented in libgomd.cpp
	*/
	// cout << "- This is libgomd instance #" << lg->m_id << endl;

	string ret = "<ERROR>";
	string omload = "";
	cout << "-- LIBGOMD: open connection to: " << remoteGomdIP << endl;
	if(lg->openConn(remoteGomdIP, atoi(remoteGomdPort.c_str()))==true)
	{
		// cout << "\n-- LIBGOMD: #1 (get nodelist)" << endl;
		ret = lg->getInfo("get nl");
		// cout << "-- LIBGOMD: value returned by the library: " << ret << endl;

		string::size_type where = ret.find(":");
		string nodelist = ret.substr(where+1);
		nodelist = nodelist.substr(0,nodelist.size()-1);
		cout << "-- nodelist  " << nodelist << endl;

		vector<string> tokens;
		Tokenize(nodelist, tokens, ",");

		// here we have the nodelist
                vector<string>::iterator p = tokens.begin();
                while (p != tokens.end()) {

			string nodeid=(string)*p;

			// here we create the widgets if we run the first time
			if (first) {
				cout << "creating the widgets" << endl;

				// the node label
				QLabel_node[h]= new QLabel(this,nodeid.c_str());
				QLabel_node[h]->setGeometry(5,labelheight,100,20);
				QLabel_node[h]->setMinimumSize(0,0);
				string nodename="node";
				nodename=nodename + nodeid;
				QLabel_node[h]->setText(nodename.c_str());
				QLabel_node[h]->setMargin(0);
				QLabel_node[h]->setAlignment( AlignVCenter | AlignCenter );
				QLabel_node[h]->setFrameStyle( QFrame::Panel | QFrame::Sunken );

				// the load progressbar
				QProgressBar_node[h]= new QProgressBar(this, nodename.c_str());
				QProgressBar_node[h]->setGeometry(115,labelheight,130,20);
				QProgressBar_node[h]->setMinimumSize(0,0);
				QProgressBar_node[h]->setTotalSteps (200);


				// adjust the height of the main widget
				myheight=myheight+25;
				this->setGeometry(20,200,300,myheight);

			}
			// here we only refresh the load values

			string getcmd="get load ";
			getcmd = getcmd + nodeid;
			omload.erase();

			// so simple we can get the load with the libgomd object ;)
			omload = lg->getInfo(getcmd);

			string::size_type position = omload.find(":");
			omload = omload.substr(position+1);
			omload = omload.substr(0, omload.size()-1);

			// here we refresh the progressbar
			cout << "-- nodeid " << nodeid << " = " << omload << endl;
			QProgressBar_node[h]->setProgress(string2int(omload));

			// increment the counters
 			p++;
			h++;
			labelheight=h*25;
		}

		first=0;


		// cout << "\n-- LIBGOMD: close connection." << endl;
		lg->closeConn();
	}



	// cout << "- Ok: exiting..." << endl;
	return (EXIT_SUCCESS);


}




